
# q-expansion of newform 858.2.i.g, downloaded from the LMFDB on 17 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 858
weight = 2
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[287, [1, 0]], [79, [1, 0]], [67, [0, -1]]]
aps_data = [[1, -1], [1, -1], [-1, 0], [0, 2], [1, -1], [1, -4], [0, -5], [0, -5], [8, -8], [2, -2], [3, 0], [-10, 10], [2, -2], [0, 4], [2, 0], [-1, 0], [0, 1], [0, 11], [-2, 2], [0, 0], [8, 0], [-4, 0], [6, 0], [-2, 2], [0, -5], [12, -12], [-4, 0], [4, -4], [19, 0], [0, -4], [-16, 16], [6, 0], [0, -10], [0, -9], [0, 4], [-20, 0], [14, 0], [0, 22], [11, -11], [0, 8], [-15, 15], [-2, 0], [0, -18], [-18, 18], [22, -22], [0, -17], [-3, 3], [-5, 5], [0, 0], [24, 0], [29, 0], [15, 0], [0, 28], [0, 24], [-18, 18], [21, -21], [0, 21], [20, -20], [0, -17], [-3, 0], [9, -9], [0, 22], [0, 0], [-22, 0], [-15, 0], [2, 0], [0, 14], [-2, 0], [0, -6], [29, -29], [-24, 24], [-23, 0], [9, -9], [0, -23], [2, -2], [0, 4], [-30, 0], [0, 18], [6, -6], [0, -30], [4, -4], [26, 0], [37, -37], [0, 21], [-24, 24], [-29, 0], [0, -12], [4, -4], [0, -10], [-21, 0], [-29, 0], [3, -3], [0, -16], [30, -30], [-14, 0], [0, -21], [6, -6], [-10, 0], [-25, 25], [42, 0], [-19, 0], [-12, 12], [0, 30], [-13, 13], [40, 0], [34, 0], [-3, 3], [-39, 0], [0, 0], [10, -10], [0, -32], [-13, 13], [0, -6], [-10, 0], [0, 9], [0, 30], [0, -8], [42, -42], [-41, 41], [0, -10], [38, -38], [20, -20], [-18, 0], [0, -44], [30, -30], [-22, 0], [0, 50], [0, 32], [16, 0], [-1, 0], [-16, 16], [-9, 9], [9, -9], [-24, 24], [0, 43], [6, -6], [0, 45], [0, 24], [0, -47], [51, -51], [27, 0], [30, -30], [0, 23], [-22, 0], [28, -28], [0, 40], [49, 0], [-11, 0], [-50, 0], [-24, 0], [0, -2], [18, -18], [-8, 0], [-24, 24], [2, -2], [10, 0], [0, 44], [0, 30], [6, 0], [48, 0], [-57, 57], [0, 30], [-50, 0], [0, 43], [-42, 42], [-10, 0], [52, -52], [0, 9]]
